#!/bin/bash

MONITOR_UNIX_PATH=""
UART_UNIX_PATH=""
RAM_SIZE=""
RAM_FILE=""
SIMPLE=0

while [[ "$#" -gt 0 ]]; do
    case $1 in
        --monitor_unix_path) MONITOR_UNIX_PATH="$2"; shift ;;
        --uart_unix_path) UART_UNIX_PATH="$2"; shift ;;
        --ram_file) RAM_FILE="$2"; shift ;;
        --ram_size) RAM_SIZE=$(($2 / 1024 / 1024)); shift ;;
        --simple) SIMPLE=1;;
        *) echo "Unknown parameter: $1" >&2; exit 1 ;;
    esac
    shift
done

if [[ ${SIMPLE} = 1 ]]; then
    MONITOR_UNIX_PATH="/tmp/monitor.unix"
    UART_UNIX_PATH="/tmp/uart.unix"
    RAM_FILE=/tmp/memory.ram
    RAM_SIZE=128
fi

if [[ -z ${MONITOR_UNIX_PATH} ]]; then
    echo "Monitor path not provided"
    exit 1
fi

if [[ -z ${UART_UNIX_PATH} ]]; then
    echo "Uart path not provided"
    exit 1
fi

if [[ -z ${RAM_SIZE} ]]; then
    echo "RAM size not provided"
    exit 1
fi

if [[ -z ${RAM_FILE} ]]; then
    echo "RAM size not provided"
    exit 1
fi

Args=(
    # Classic qemu setup for kernel challenges
    -kernel bzImage
    -initrd initramfs.cpio
    -nographic
    -display none
    -no-reboot
    -vga none
    -net none
    -snapshot
    -fw_cfg name=opt/flag,file=flag.txt

    # Memory setup
    # This is necessary to allow aliasing the RAM into two physical locations while still treating the
    # memory as separate QEMU memory regions.
    -object memory-backend-file,size=${RAM_SIZE}m,id=memory.ram,mem-path=${RAM_FILE},share=on,prealloc=on
    -m ${RAM_SIZE}m
    -M memory-backend=memory.ram

    # Kernel arguments
    # mem_encrypt=on is necessary to enable SME
    -append "console=ttyS0 mem_encrypt=on nokaslr"

    # Level 0x8000001f is necessary to allow querying SME
    -cpu "hxp-hack1"

    # The monitor is used for reading / writing memory
    -monitor unix:${MONITOR_UNIX_PATH},server,nowait

    # Makes it uniform for handling IO with the frontend app
    -serial unix:${UART_UNIX_PATH},server,nowait

    # Enable debugging
    # -s
)

./qemu_system_x86_64_sme "${Args[@]}"
